from rfc3339 import convert_datetime  # noqa
from tasks import set_task_percentage  # noqa
from metadata import Metadata  # noqa
from connections import Connections  # noqa
from image_detail import ImageDetail, ImageDetailMap  # noqa
from ip_address import IpAddress  # noqa
from locations import Region  # noqa
from resource import Resource  # noqa
from appliance import Appliance  # noqa
from network import Network  # noqa
from server import Server  # noqa
from volume import Volume  # noqa
from tools import serialize_list  # noqa



from datetime import datetime
import six
from . import Connections
from . import Metadata
from . import convert_datetime


class Resource(object):
    def __init__(self, id=None, name=None, type=None, region=None,
                 provider_created_at=None, metadata=None,
                 native_portal_link=None, connections=None):
        self.id = id
        self.name = name
        self.type = type
        self._check_not_none_str_value('id', self.id)
        self._check_not_none_str_value('name', self.name)
        self._check_not_none_str_value('type', self.type)

        self.provider_created_at = provider_created_at

        if connections is None:
            connections = Connections()
        self.connections = connections

        if metadata is None:
            metadata = Metadata()
        self.metadata = metadata
        self.region = region
        self.native_portal_link = native_portal_link

    def _check_not_none_str_value(self, name, value):
        if value is None or not isinstance(value, six.string_types):
            raise ValueError('%s must be a string and have a value' % name)

    def serialize(self):
        provider_created_at = convert_datetime(self.provider_created_at)

        regions = []
        if self.region is not None:
            regions = [self.region.serialize()]

        results = {
            "id": self.id,
            "type": self.type,
            "base": {
                "name": self.name,
                "regions": regions,
                "provider_created_at": provider_created_at,
                'last_seen_at': convert_datetime(datetime.utcnow()),
            },
            "connections": self.connections.serialize(),
            "metadata": self.metadata.serialize(),
        }

        if self.native_portal_link is not None:
            results['base']['native_portal_link'] = self.native_portal_link

        return results



from . import Resource


class Network(Resource):
    def __init__(self, **kwargs):
        super(Network, self).__init__(type='network', **kwargs)

    def serialize(self):
        data = super(Network, self).serialize()
        data['details'] = {'network': {}}
        return data



from . import Resource


class Volume(Resource):
    def __init__(self, size_b=None, encrypted=None, iops=None, zone_name=None,
                 **kwargs):
        super(Volume, self).__init__(type='volume', **kwargs)
        self.size_b = size_b
        self.encrypted = encrypted
        self.iops = iops
        self.zone_name = zone_name

    def serialize(self):
        data = super(Volume, self).serialize()

        iops = None
        if self.iops is not None:
            iops = int(self.iops)

        data['details'] = {
            self.type: {
                "encrypted": self.encrypted,
                "iops": iops,
                "size_b": self.size_b,
                "zone_name": self.zone_name,
            }
        }
        return data



class Connections(object):
    def __init__(self):
        self.data = {}

    def _add_type(self, type_, connections):
        if connections is None or len(connections) == 0:
            return

        if type_ not in self.data:
            self.data[type_] = []

        type_data = self.data[type_]
        for id_ in connections:
            type_data.append({'id': id_})

    def add(self, type=None, connections=None, appliances=None, servers=None,
            networks=None, volumes=None):
        if type is not None and connections is not None:
            self._add_type(type, connections)

        self._add_type('appliance', appliances)
        self._add_type('server', servers)
        self._add_type('network', networks)
        self._add_type('volume', volumes)

        return self

    def serialize(self):
        return self.data



from pip.req import parse_requirements
from setuptools import setup, find_packages
import uuid

requirements = parse_requirements('requirements.txt', session=uuid.uuid1())
install_requires = [str(r.req) for r in requirements]
description = (
    "nflex-connector-utils provides a suite of tools to produce data "
    "structures when implementing resources connector nflex modules.")

with open('README.md') as f:
    long_description = f.read()

setup(
    name="nflex-connector-utils",
    version="0.1.2",
    description=description,
    long_description=long_description,
    url='http://www.ntt.com',
    author='NTT communications',
    licence='GNU General Public License v2 (GPLv2)',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v2 (GPLv2)',
        'Natural Language :: English',
        'Programming Language :: Python :: 2 :: Only',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development',
    ],
)
